<?php

namespace App\Models;

use App\Repositories\CategoryRepository;
use App\Traits\Taggable;
use Illuminate\Database\Eloquent\Model;
use Cviebrock\EloquentSluggable\Sluggable;

class Category extends Model
{

    protected $guarded = ['id'];
    public function categories()
    {
        return $this->hasMany(Category::class)->orderBy('ordering');
    }

    public function childrenCategories()
    {
        return $this->hasMany(Category::class)->with('categories')->orderBy('ordering');
    }

    public function allChildCategories()
    {
        $categories = $this->categories()->pluck('id')->toArray();
        $categories[] = $this->id;

        foreach ($this->categories as $category) {
            $categories = array_merge($categories, $category->allChildCategories());
        }

        return $categories;
    }

    public function allProducts()
    {
        $categories = $this->allChildCategories();

        $products = Product::whereHas('categories', function ($query) use ($categories) {
            $query->whereIn('categories.id', $categories);
        });

        return $products;
    }

    public function children()
    {
        return $this->child()->with('children');
    }

    public function descendants()
    {
        $collection = new \Illuminate\Support\Collection();

        foreach ($this->child as $chi) {
            $collection->add($chi);
            $collection = $collection->merge($chi->descendants());
        }

        return $collection;
    }

    public function allPublishedProducts()
    {
        $categories = $this->allChildCategories();

        $products = Product::published()->whereHas('categories', function ($query) use ($categories) {
            $query->whereIn('categories.id', $categories)->published();
        });

        return $products;
    }

    public function productsCount()
    {
        return $this->allProducts()->count();
    }

    public function menus()
    {
        return $this->belongsToMany(Menu::class, 'menus', 'menuable_id')->where('type', 'category');
    }

    public function getFullTitleAttribute()
    {
        if ($this->category_id) {
            $category_id = $this->category_id;
            $value = [$this->title];
            do {
                $mother = Category::find($category_id);
                $value[] = $mother->title;
                $category_id = $mother->category_id;
            } while ($category_id);

            $value = array_reverse($value);

            return implode(' - ', $value);
        }

        return $this->title;
    }

    public function parent()
    {
        return $this->belongsTo(Category::class, 'category_id');
    }

    public function child()
    {
        return $this->hasMany(Self::class, 'category_id', 'id');
    }

    public function parents()
    {
        $parents = collect([]);

        $parent = $this->parent;

        while (!is_null($parent)) {
            $parents->push($parent);
            $parent = $parent->parent;
        }

        return $parents->reverse();
    }

    public function products()
    {
        return $this->hasMany(Product::class);
    }



    public function scopeGetWithChilds($query)
    {
        $categories     = $query->get();
        $all_categories = Category::get();

        $this->setCategoriesRelations($categories, $all_categories);

        return $categories;
    }

    private function setCategoriesRelations($categories, $all_categories)
    {
        foreach ($categories as $category) {
            if ($all_categories->where('category_id', $category->id)->count()) {
                $category->setRelation('categories', $all_categories->where('category_id', $category->id));
                $this->setCategoriesRelations($all_categories->where('category_id', $category->id), $all_categories);
            }
        }
    }

    public function scopePublished($query)
    {
        return $query->where('published', true);
    }

    public function isPublished()
    {
        return $this->published;
    }
}
